import datetime
from django.contrib.auth.models import Permission
from django_dynamic_fixture import G
from django_webtest import WebTest
from itertools import count
from ereports.engine.config import reportform_factory
from ereports.engine.datasource import Datasource
from ereports.engine.renderer import BaseHtmlRender, BaseXlsRender
from ereports.engine.report import BaseReport, BaseGrouper, ImproperlyConfigured
from ereports.tests.app.models import SimpleDemoModel, SimpleDateModel
from ereports.tests.app.reports import SimpleDemoReport, SimpleDateReport, SimpleDateModelSource


class SequentialDataFixture(object):
    def __init__(self, start_from):
        super(SequentialDataFixture, self).__init__()
        self.gen = count(start_from)

    def generate_data(self, field):
        return self.gen.next()


def fake_callable(orig):
    return u'cba'


class TestBaseGroup(WebTest):
    def test_init(self):
        g = BaseGrouper(SimpleDemoReport, 'char', 'Character')
        self.assertEqual(g.report, SimpleDemoReport)
        self.assertEqual(g.group_by, 'char')
        self.assertEqual(g.internal_order, 'Character')
        self.assertFalse(g._processed)

    def test_items_basestring(self):
        G(SimpleDateModel, char='abc', date=datetime.date.today(), date_range=datetime.date.today())
        r = SimpleDateReport.as_report()
        g = BaseGrouper(r, 'char', 'Character')
        self.assertEqual(list(g.items()), [(u'abc', list(r))])
        self.assertTrue(g._processed)

        # call again to ensure _processed param handled properly and no error
        g._process()
        self.assertTrue(g._processed)

    def test_items_callable(self):
        G(SimpleDateModel, char='abc', date=datetime.date.today(), date_range=datetime.date.today())
        r = SimpleDateReport.as_report()
        g = BaseGrouper(r, fake_callable, 'Groups')
        self.assertEqual(list(g.items()), [(u'cba', list(r))])

    def test_items_other(self):
        G(SimpleDemoModel, char='abc')
        r = SimpleDemoReport.as_report()
        r.list_display = ['char', 'integer1']
        g = BaseGrouper(r, 'wrong', 'wrong')
        self.assertEqual(list(g.items()), [(None, list(r))])

    def test_items_sorted(self):
        G(SimpleDateModel, char='cba', date=datetime.date.today(), date_range=datetime.date.today())
        G(SimpleDateModel, char='bac', date=datetime.date.today(), date_range=datetime.date.today())
        G(SimpleDateModel, char='abc', date=datetime.date.today(), date_range=datetime.date.today())
        r = SimpleDateReport.as_report()
        g = BaseGrouper(r, 'char', 'Groups')
        expected = [(u'abc', [r[2]]),
                    (u'bac', [r[1]]),
                    (u'cba', [r[0]])]

        self.assertEqual(list(g.items()), expected)

    def test_values(self):
        G(SimpleDateModel, char='abc', date=datetime.date.today(), date_range=datetime.date.today())
        r = SimpleDateReport.as_report()
        g = BaseGrouper(r, 'char', 'Groups')
        self.assertEqual(g.values(), [list(r)])

    def test_keys(self):
        G(SimpleDateModel, char='abc', date=datetime.date.today(), date_range=datetime.date.today())
        r = SimpleDateReport.as_report()
        g = BaseGrouper(r, 'char', 'Groups')
        self.assertEqual(g.keys(), [u'abc'])


class TestBaseReport(WebTest):
    def test_inherit(self):
        TestReport = type('TestReport', (BaseReport,), {'model': Permission})
        r = TestReport.as_report()
        self.assertIsInstance(list(r), list)

    def test_repr(self):
        r = SimpleDemoReport.as_report()
        self.assertEqual(repr(r), "<SimpleDemoReport: on `SimpleDemoModel`>")

    def test_type_error_exception(self):
        with self.assertRaises(TypeError):
            BaseReport.as_report(models="")

    def test_improperly_configured_exception(self):
        G(SimpleDemoModel, n=2, char='abc', integer1=10, integer2=20)
        with self.assertRaises(ImproperlyConfigured):
            BaseReport.as_report(models=SimpleDemoModel)

    def test_slice(self):
        instances = G(SimpleDemoModel, n=2, char='abc', integer1=10, integer2=20, boolean=True)
        r = BaseReport.as_report(model=SimpleDemoModel)
        self.assertSequenceEqual(list(r[:1]), [(instances[0].pk, u'abc', 10, 20, True)])

    def test_getitem(self):
        instances = G(SimpleDemoModel, n=2, char='abc', integer1=10, integer2=20, boolean=True)
        r = BaseReport.as_report(model=SimpleDemoModel)
        self.assertSequenceEqual(r[1].values(), (instances[1].pk, u'abc', 10, 20, True))

    def test_datasource(self):
        instances = G(SimpleDemoModel, n=2, char='abc', integer1=10, integer2=20, boolean=True)
        ds = Datasource.as_datasource(model=SimpleDemoModel)
        r = BaseReport.as_report(datasource=ds)
        self.assertSequenceEqual(r[1].values(), (instances[1].pk, u'abc', 10, 20, True))

    def test_datasource_custom_list_display(self):
        G(SimpleDemoModel, n=2, char='abc', integer1=10, integer2=20)

        ds = Datasource.as_datasource(model=SimpleDemoModel)
        r = BaseReport.as_report(datasource=ds,
                                 list_display=['integer2', 'char', 'integer1'])

        self.assertSequenceEqual(r[1], (20, u'abc', 10))
        self.assertSequenceEqual(r[:1], [(20, u'abc', 10)])

    def test_datasource_std_list_display(self):
        G(SimpleDemoModel, n=2, char='abc', integer1=10, integer2=20)

        ds = Datasource.as_datasource(model=SimpleDemoModel,
                                      columns=['integer2', 'char', 'integer1'])
        r = BaseReport.as_report(datasource=ds)

        self.assertSequenceEqual([c.name for c in ds.columns], r.display_order())
        self.assertSequenceEqual(r[1], (20, u'abc', 10))

    def test_headers(self):
        G(SimpleDemoModel, n=2, char='abc', integer1=10, integer2=20)

        ds = Datasource.as_datasource(model=SimpleDemoModel)
        r = BaseReport.as_report(datasource=ds,
                                 list_display=['integer2', 'char', 'integer1'])

        self.assertSequenceEqual(['Integer #2', 'Character', 'Integer #1'], r.headers)

        ds = Datasource.as_datasource(model=SimpleDemoModel,
                                      columns=['integer2', 'char', 'integer1'])
        r = BaseReport.as_report(datasource=ds)

        self.assertSequenceEqual(['Integer #2', 'Character', 'Integer #1'], r.headers)

    def test_get_column_values(self):
        G(SimpleDemoModel, n=10, data_fixture=SequentialDataFixture(0))
        ds = Datasource.as_datasource(model=SimpleDemoModel)
        r = BaseReport.as_report(datasource=ds,
                                 list_display=['integer2', 'char', 'integer1'])

        self.assertSequenceEqual([1, 3, 5, 7, 9, 11, 13, 15, 17, 19], r.get_column_values('integer1'))

    def test_get_config_form_class(self):
        r = SimpleDemoReport.as_report()
        f = reportform_factory(r, bases=(r.config_form_class,))
        self.assertIn('integer1', f.base_fields)
        self.assertIn('_format', f.base_fields)

    def test_get_format_labels(self):
        r = SimpleDemoReport.as_report()
        self.assertEqual(r.formats_dict, {'html': BaseHtmlRender, 'xls': BaseXlsRender})

    def test_format_labels(self):
        r = SimpleDemoReport.as_report()
        self.assertSequenceEqual(sorted(r.get_format_labels()), ['html', 'xls'])

    def test_get_renderer_class_format(self):
        r = SimpleDemoReport.as_report()
        self.assertEqual(r.get_renderer_class_for_format('html'), BaseHtmlRender)

    def test_get_renderer_for_format(self):
        r = SimpleDemoReport.as_report()
        renderer = r.get_renderer_for_format('html')
        self.assertIsInstance(renderer, BaseHtmlRender)

    def test_get_groups(self):
        r = SimpleDemoReport.as_report()

        with self.assertRaises(ImproperlyConfigured):
            r.get_groups()

        G(SimpleDateModel, char='abc', date=datetime.date.today(), date_range=datetime.date.today())
        r = SimpleDateReport.as_report(datasource=SimpleDateModelSource.as_datasource())
        g = r.get_groups()
        self.assertEqual(len(g), 1)

    def test_has_subtotals(self):
        r = SimpleDemoReport.as_report()

        self.assertFalse(r.has_subtotals())

        r.column_totals = ['integer1']
        self.assertTrue(r.has_subtotals())


class TestExtendedReport(WebTest):
    def test_datasource_custom_list_display(self):
        G(SimpleDemoModel, n=2, char='abc', integer1=10, integer2=20)
        r = SimpleDemoReport.as_report()
        self.assertEqual(r.datasource[1].values(), [u'abc', 10])
        self.assertEqual(r[1], (u'abc',))
        self.assertSequenceEqual(r[:1], [(u'abc',)])



# -*- coding: utf-8 -*-
from django.conf import settings
from django.template import TemplateDoesNotExist
from django.template.loader import BaseLoader

from pasportng.ereports.models import ReportTemplate


class EReportTemplateLoader(BaseLoader):
    is_usable = True

    def load_template_source(self, template_name, template_dirs=None):
        try:
            tpl = ReportTemplate.objects.get(name=template_name)
            return (unicode(tpl.body).decode(settings.FILE_CHARSET), template_name)
        except ReportTemplate.DoesNotExist:
            raise TemplateDoesNotExist(template_name)
    load_template_source.is_usable = True

_loader = EReportTemplateLoader()



from django.contrib.auth.models import User
from django.db import models
import itertools


class DemoModelGroup(models.Model):
    name = models.CharField(max_length=255)
    user = models.ForeignKey(User)

    def __unicode__(self):
        return self.name


class DemoOptionalModel(models.Model):
    name = models.CharField(max_length=255)
    user = models.ForeignKey(User, null=True)


class DemoModelDetail(models.Model):
    name = models.CharField(max_length=255)


counter = itertools.count()


class SimpleDemoModel(models.Model):
    char = models.CharField('Character', max_length=255)
    integer1 = models.IntegerField('Integer #1')
    integer2 = models.IntegerField('Integer #2', null=True, blank=True, default=0)
    boolean = models.BooleanField('Boolean', default=False)

    def filter_source(self):
        return counter.next()


class SimpleDateModel(models.Model):
    char = models.CharField('Character', max_length=255)
    date = models.DateField('Date')
    date_range = models.DateField('Date Range')


class DemoModel(models.Model):
    group = models.ForeignKey(DemoModelGroup)
    m2m = models.ManyToManyField(DemoModelDetail)

    char = models.CharField(max_length=255)
    integer = models.IntegerField()
    logic = models.BooleanField()
    null_logic = models.NullBooleanField()
    date = models.DateField()
    datetime = models.DateTimeField()
    time = models.TimeField()
    decimal = models.DecimalField(max_digits=10, decimal_places=3)
    email = models.EmailField()
    float = models.FloatField()
    bigint = models.BigIntegerField()
    ip = models.IPAddressField()
    url = models.URLField()
    text = models.TextField()

    nullable = models.CharField(max_length=255, null=True, default=None)
    blank = models.CharField(max_length=255, blank=True, null=False, default="")
    not_editable = models.CharField(max_length=255, editable=False, blank=True, null=True)
    choices = models.IntegerField(choices=((1, 'Choice 1'), (2, 'Choice 2'), (3, 'Choice 3')))

    class Meta:
        app_label = 'app'
        db_table = 'ereports_demoapp_demomodel'

    def get_attribute_getter(self):
        return "getter"

    @property
    def attribute_property(self):
        return "property"



from django import forms
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.forms import ChoiceField
from django.forms.models import ModelForm
from ereports.models import ReportConfiguration


class MyModelChoiceField(forms.ModelChoiceField):
    def label_from_instance(self, obj):
        return "%s - %s" % (obj.app_label.title(), obj.name.title(),)


class ReportConfigurationForm(ModelForm):
    target_model = MyModelChoiceField(queryset=ContentType.objects.order_by('app_label', 'name'), required=False)
    report_class = ChoiceField()

    def __init__(self, *args, **kwargs):
        from ereports.engine.registry import registry
        super(ReportConfigurationForm, self).__init__(*args, **kwargs)

        self.fields['report_class'].choices = registry.choices()

    def clean_groupby(self):
        value = self.cleaned_data['groupby']
        for line in value.splitlines():
            if len(line.split(';')) != 2:
                raise ValidationError('wqww')
        return value

    class Meta:
        model = ReportConfiguration



from __future__ import absolute_import
from decimal import Decimal, InvalidOperation
from django import template
from ereports.engine.columns import RowValue, RowValueError
from ereports.utils import currencyformat, fqn

register = template.Library()


def isNumeric(s):
    try:
        return int(s) or True
    except (ValueError, TypeError):
        pass
    try:
        Decimal(s) or True
    except (ValueError, TypeError, InvalidOperation):
        pass
    try:
        return float(s) or True
    except (ValueError, TypeError):
        pass
    return False

register.filter(currencyformat)


@register.filter
def col_to_css_class(row_value):
    if isinstance(row_value, RowValue):
        return fqn(row_value.column).lower().replace('.', '_')
    elif isinstance(row_value, RowValueError):
        return str(row_value)
    else:
        raise ValueError('wrong value for `col_to_css_class` ({0}, {1})'.format(row_value, type(row_value)))


@register.filter
def with_widget(row_value, format=None):
    if isinstance(row_value, RowValue):
        widget = row_value.column.widget
        render = getattr(widget, 'render_{}'.format(format), widget.render)
        return render(row_value)
    elif isinstance(row_value, RowValueError):
        return str(row_value)
    else:
        raise ValueError('wrong value for `with_widget` ({0})'.format(type(row_value)))


@register.filter
def humanize_filters(filters):
    LOOKUP_MAPPING = [
        'exact',
        'iexact',
        'contains',
        'icontains',
        'in',
        'gt',
        'gte',
        'lt',
        'lte',
        'startswith',
        'istartswith',
        'endswith',
        'iendswith',
        'range',
        'year',
        'month',
        'day',
        'week_day',
        'isnull',
        'search',
        'regex',
        'iregex',
    ]

    def get_label(parts):
        if parts[-1] in LOOKUP_MAPPING:
            return " ".join(parts[-2:])
        return parts[-1]

    ret = []
    if filters:
        for k, v in filters.items():
            if k:
                parts = k.lower().split('__')
                label = get_label(parts)
                last = parts[-1]
                # if not k.lower().endswith('office'):
                if last != 'office':
                    ret.append((label.replace('_', ' ').title(), str(v)))
    return sorted(ret)


@register.filter
def format_raw_value(row_value):
    if isinstance(row_value.value, Decimal):
        return currencyformat(row_value.value)
    return row_value.value


@register.simple_tag()
def subtotal(report, rows, col):
    try:
        if isinstance(col, basestring):
            col = report.get_column_by_name(col)
        tot = sum(r[col.name].value or 0 for r in rows)
        return format_raw_value(RowValue(tot, col))
    except TypeError:
        return 0


@register.simple_tag()
def total(report, col):
    try:
        if isinstance(col, basestring):
            col = report.get_column_by_name(col)
        values = report.get_column_values(col.name)
        return format_raw_value(RowValue(sum(values), col))
    except TypeError:
        return 0


@register.filter
def div_mod(dividend, divisor):
    quotient, remainder = divmod(dividend, divisor)
    return quotient, remainder



# -*- encoding: utf-8 -*-
import random
import string
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.db import models


clean_field = lambda field: field.replace('.', '__').strip()
keygen = lambda: ''.join(random.choice(string.ascii_uppercase + string.digits) for x in range(100))


class ReportGroup(models.Model):
    name = models.CharField(max_length=100, unique=True)
    parent = models.ForeignKey('self', blank=True, null=True)

    class Meta:
        app_label = 'ereports'

    def __unicode__(self):
        return self.name


class ReportTemplate(models.Model):
    name = models.CharField(max_length=100, unique=True)
    body = models.TextField()
    system = models.BooleanField(default=False)

    class Meta:
        app_label = 'ereports'

    def __unicode__(self):
        return self.name


def validate_report_class(value):
    from ereports.engine.registry import registry

    if value not in registry:
        raise ValidationError(u'%s is not a registered Report class ' % value)


class ReportConfiguration(models.Model):
    name = models.CharField(max_length=100, unique=True)
    group = models.ForeignKey(ReportGroup)
    template = models.ForeignKey(ReportTemplate, blank=True, null=True)
    description = models.CharField(max_length=2000, blank=True, null=True)

    report_class = models.CharField(max_length=200, validators=[validate_report_class])

    target_model = models.ForeignKey(ContentType, blank=True, null=True)
    # datasource = models.CharField('PostProcessor', max_length=200, blank=True, null=True,
    #                               choices=[(k, k) for k, v in postprocessor.registry.iteritems()])

    published = models.BooleanField(default=False)
    select_related = models.BooleanField(default=False)
    use_distinct = models.BooleanField(default=False)

    columns = models.TextField(default='id\n')
    filtering = models.TextField(blank=True, null=True, default="")
    ordering = models.TextField(blank=True, null=True,
                                help_text="allowed ordering for the report. use comma to split fields and semicolon to "
                                          "split the label, ie `employee.lastname,employee.first_name; Name`")

    groupby = models.TextField(blank=True, null=True,
                               help_text="allowed groupby for the report. use semicolon to "
                                         "indicate field;label, ie `contract.type;Contract Group`")

    # extras = models.TextField(blank=True, null=True)
    # rawsql = models.TextField(blank=True, null=True)

    ttl = models.IntegerField('Cache validity (minutes)', default=0)
    cache_key = models.CharField('Cache entry seed', max_length=200, null=False, unique=True, default=keygen)

    class Meta:
        permissions = (('run_report', 'Can run a report'),
                       ('read_report', 'Can read a report'),)
        app_label = 'ereports'

    def __unicode__(self):
        return self.name

    def title(self):
        return self.name

    def get_hard_filters(self, context):
        filters = {}
        if self.filtering:
            for line in self.filtering.splitlines():
                if '=' in line:
                    key, value = line.strip().split('=')
                    if value.startswith('{{'):
                        value = context[value[2:-2].strip()]
                        filters[key.replace('.', "__")] = value
        return filters

    def get_allowed_group_by(self):
        grouping = []
        if self.groupby:
            for g in self.groupby.strip().splitlines():
                o = g.split(';', 1)
                grouping.append((o[0].strip(), o[-1].strip()))
        return grouping

    def get_allowed_order_by(self):
        order_clauses = []
        if self.ordering:
            for g in self.ordering.strip().splitlines():
                o = g.split(';', 1)
                order_clauses.append((o[0].strip(), o[-1].strip()))
        return order_clauses

    def get_allowed_filters(self):
        filters = []
        if self.filtering:
            for line in self.filtering.strip().splitlines():
                if '=' not in line:
                    filters.append(line.strip())
        return filters



from django.conf import settings
import os
import sys


def pytest_configure(config):
    here = os.path.dirname(__file__)
    sys.path.insert(0, here)

    if not settings.configured:
        os.environ['DJANGO_SETTINGS_MODULE'] = 'ereports.tests.settings'



import pickle
from django.contrib.contenttypes.models import ContentType
from django_dynamic_fixture import G
import mock
from django.core import cache
from django.contrib.auth.models import Permission
from django.test.testcases import TestCase, _AssertNumQueriesContext
from ereports.engine.cache import monitor_model, reset, DatasourceCacheManager
from ereports.engine.datasource import Datasource
from django.db import connections
from ereports.engine.report import BaseReport
from ereports.tests.app.models import SimpleDemoModel

locmem_cache = cache.get_cache('django.core.cache.backends.locmem.LocMemCache')
dummy_cache = cache.get_cache('django.core.cache.backends.dummy.DummyCache')


class TestDatasourceCacheManager(TestCase):
    def test_get_key(self):
        manager = DatasourceCacheManager()
        dummy_cache = cache.get_cache('django.core.cache.backends.dummy.DummyCache')
        dummy_cache.clear()
        with mock.patch('ereports.engine.cache._cache', dummy_cache):
            monitor_model(Permission)
            reset(Permission)

            TestDatasource = type('TestDatasource', (Datasource,), {'model': Permission,
                                                                    'dependent_models': [ContentType],
                                                                    'use_cache': True,
                                                                    'columns': ['id', 'name']})
            ds1 = TestDatasource.as_datasource()
            k = manager.get_key(ds1)
            self.assertTrue(k.startswith('ereports/1/django.contrib.auth.models.Permission/0'
                                         '/django.contrib.contenttypes.models.ContentType/0/'), k)

    def test_generation(self):
        locmem_cache = cache.get_cache('django.core.cache.backends.locmem.LocMemCache')
        manager = DatasourceCacheManager()
        with mock.patch('ereports.engine.cache._cache', locmem_cache):
            monitor_model(Permission)
            reset(Permission)

            TestDatasource = type('TestDatasource', (Datasource,),
                                  {'model': Permission,
                                   'dependent_models': [ContentType],
                                   'use_cache': True,
                                   'columns': ['id', 'name']})

            ds1 = TestDatasource.as_datasource()
            k1 = manager.get_key(ds1)
            self.assertTrue(k1.startswith('ereports/1/django.contrib.auth.models.Permission/0'
                                          '/django.contrib.contenttypes.models.ContentType/0/'), k1)
            Permission.objects.filter(pk=1).delete()
            k2 = manager.get_key(ds1)
            self.assertTrue(k2.startswith('ereports/1/django.contrib.auth.models.Permission/1'
                                          '/django.contrib.contenttypes.models.ContentType/0/'), k2)


class TestCacheDatasource(TestCase):
    def test_queryset_no_hit_db(self):
        TestDatasource = type('TestDatasource', (Datasource,), {'model': Permission,
                                                                'use_cache': False,
                                                                'columns': ['id', 'name']})
        ds = TestDatasource.as_datasource()
        with _AssertNumQueriesContext(self, 0, connections['default']):
            ds._get_queryset()

    def test_queryset_cache(self):
        with mock.patch('ereports.engine.cache._cache', locmem_cache):
            locmem_cache.clear()

            TestDatasource = type('TestDatasource', (Datasource,), {'model': Permission,
                                                                    'use_cache': True,
                                                                    'columns': ['id', 'name']})
            ds = TestDatasource.as_datasource()
            with _AssertNumQueriesContext(self, 1, connections['default']):
                i1 = list(ds)
            with _AssertNumQueriesContext(self, 0, connections['default']):
                i2 = list(ds)
            self.assertSequenceEqual(i1, i2)

    def test_pickle(self):
        TestDatasource = type('TestDatasource', (Datasource,), {'model': Permission,
                                                                'use_cache': True,
                                                                'columns': ['id', 'name']})
        ds1 = TestDatasource.as_datasource()
        self.assertTrue(pickle.dumps(ds1.get_data()))

    def test_base_cache(self):
        with mock.patch('ereports.engine.cache._cache', locmem_cache):
            locmem_cache.clear()
            TestDatasource = type('TestDatasource', (Datasource,), {'model': Permission,
                                                                    'use_cache': True,
                                                                    'columns': ['id', 'name']})
            from ereports.engine.cache import _cache

            _cache.set('aa', 1)
            assert _cache.get('aa') == 1

            ds1 = TestDatasource.as_datasource()
            ds2 = TestDatasource.as_datasource()

            with _AssertNumQueriesContext(self, 1, connections['default']):
                i1 = list(ds1)
                self.assertSequenceEqual(i1, list(ds2))

        with mock.patch('ereports.engine.cache._cache', dummy_cache):
            TestDatasource = type('TestDatasource', (Datasource,), {'model': Permission,
                                                                    'use_cache': True,
                                                                    'columns': ['id', 'name']})
            ds1 = TestDatasource.as_datasource()
            ds2 = TestDatasource.as_datasource()

            with _AssertNumQueriesContext(self, 2, connections['default']):
                i1 = list(ds1)
                self.assertSequenceEqual(i1, list(ds2))

    def test_cache_different_columns(self):
        with mock.patch('ereports.engine.cache._cache', locmem_cache):
            locmem_cache.clear()

            TestDatasource = type('TestDatasource', (Datasource,), {'model': Permission, 'use_cache': True})
            ds1 = TestDatasource.as_datasource(columns=['id', 'name'])
            ds2 = TestDatasource.as_datasource(columns=['id'])

            with _AssertNumQueriesContext(self, 2, connections['default']):
                list(ds1)
                list(ds2)

    def test_cache_different_filters(self):
        with mock.patch('ereports.engine.cache._cache', locmem_cache):
            locmem_cache.clear()

            TestDatasource = type('TestDatasource', (Datasource,), {'model': Permission,
                                                                    'use_cache': True,
                                                                    'columns': ['id', 'name']})
            ds1 = TestDatasource.as_datasource()
            ds1.add_filters(id__gt=0)
            ds2 = TestDatasource.as_datasource()

            with _AssertNumQueriesContext(self, 2, connections['default']):
                i1 = list(ds1)
                i2 = list(ds2)
                self.assertSequenceEqual(i1, i2)

    def test_cache_invalidate_on_update(self):
        with mock.patch('ereports.engine.cache._cache', locmem_cache):
            locmem_cache.clear()
            monitor_model(Permission)
            reset(Permission)
            TestDatasource = type('TestDatasource', (Datasource,), {'model': Permission,
                                                                    'use_cache': True,
                                                                    'columns': ['id', 'name']})
            ds1 = TestDatasource.as_datasource()
            list(ds1)
            p = Permission.objects.get(pk=1)
            p.name = 'aaaaaa'
            p.save()
            ds2 = TestDatasource.as_datasource()
            with _AssertNumQueriesContext(self, 1, connections['default']):
                list(ds2)

    def test_cache_invalidate_on_delete(self):
        with mock.patch('ereports.engine.cache._cache', locmem_cache):
            locmem_cache.clear()
            monitor_model(Permission)
            reset(Permission)

            TestDatasource = type('TestDatasource', (Datasource,), {'model': Permission,
                                                                    'use_cache': True,
                                                                    'columns': ['id', 'name']})
            ds1 = TestDatasource.as_datasource()
            list(ds1)
            Permission.objects.filter(pk=1).delete()
            ds2 = TestDatasource.as_datasource()
            with _AssertNumQueriesContext(self, 1, connections['default']):
                list(ds2)


class TestCacheReport(TestCase):
    def test_cache(self):
        locmem_cache = cache.get_cache('django.core.cache.backends.locmem.LocMemCache')
        # dummy_cache = cache.get_cache('django.core.cache.backends.dummy.DummyCache')
        locmem_cache.clear()
        with mock.patch('ereports.engine.cache._cache', locmem_cache):
            G(SimpleDemoModel, n=2, char='abc', integer1=10, integer2=20, boolean=True)
            r1 = BaseReport.as_report(model=SimpleDemoModel, use_cache=True)
            r2 = BaseReport.as_report(model=SimpleDemoModel, use_cache=True)

            with _AssertNumQueriesContext(self, 1, connections['default']):
                i1 = list(r1)
                self.assertSequenceEqual(i1, list(r2))



# -*- encoding: utf-8 -*-

from django.conf.urls import patterns, url
from .views import ReportIndex, ReportFilter


urlpatterns = patterns(
    '',

    url(r'^$', ReportIndex.as_view(), name='ereports.list'),
    url(r'report/(?P<pk>\d+)/$', ReportFilter.as_view(), name='ereports.filter_report'),
)



import datetime
from django.db.models import Sum
from django.db.models.query import EmptyQuerySet, QuerySet
from kiowa.utils import get_attr


registry = {}


class FakeQuerySet(EmptyQuerySet):
    """Turn a list into a Django QuerySet... kind of."""
    def __init__(self, model=None, query=None, using=None, items=[]):
        super(FakeQuerySet, self).__init__(model, query, using)
        self._result_cache = items

    def count(self):
        return len(self)


class SubTotalGroup(list):

    def __getattr__(self, fieldname):
        tot = 0
        for el in self:
            attr = get_attr(el, fieldname)
            if callable(attr):
                tot += attr()
            else:
                tot += attr or 0
        return tot


class SubTotalList(object):
    def __init__(self, *groups):
        self.groups = groups
        self._entries = {}

    def __getitem__(self, y):
        if not y in self._entries:
            self._entries[y] = SubTotalGroup()
        return self._entries[y]

    def get_unique_key_for_target(self, target):
        return "_".join([str(get_attr(target, g)) for g in self.groups])

    def process(self, target):
        hash = self.get_unique_key_for_target(target)
        self[hash].append(target)
        return self[hash]

    def __repr__(self):
        return "<Subtotal on %s >" % repr(self.groups)


class PostProcessor(QuerySet):
    allow_filtering = True
    allow_grouping = True
    allow_grouping = True

    def __init__(self, model=None, query=None, using=None, report=None, queryset=None):
        self._report = report
        self._queryset = queryset
        super(PostProcessor, self).__init__(model, query, using)

    def _clone(self, klass=None, setup=False, **kwargs):
        if klass is None:
            klass = self.__class__
        query = self.query.clone()
        if self._sticky_filter:
            query.filter_is_sticky = True
        c = klass(model=self.model, query=query, using=self._db, report=self._report, queryset=self._queryset)
        c._for_write = self._for_write
        c._prefetch_related_lookups = self._prefetch_related_lookups[:]
        c.__dict__.update(kwargs)
        if setup and hasattr(c, '_setup_query'):
            c._setup_query()
        return c


class SubTotalPostProcessor(PostProcessor):
    def all(self):
        processed = {}
        x = self._clone()
        if self._report.groupby:
            total = SubTotalList(*self._report.get_target_grouping())
            for el in x:
                unique_key = total.get_unique_key_for_target(el)
                el.subtotal_group = total.process(el)
                processed[unique_key] = el.pk

        return filter(lambda x: x.id not in processed.values(), x)


class ContractPostProcessor(PostProcessor):

    def filter(self, *args, **kwargs):
        return self._clone()

    def active(self, *args, **kwargs):
        return self._clone()

    def inactive(self, *args, **kwargs):
        from pasportng.pasport.models import Contract
        return Contract.objects.latest_contracts(*args, **kwargs)


class LeavePostProcessor(PostProcessor):
    def all(self):
        _all = self._clone()
        for contract in _all:
            deadline = datetime.date.today() if not contract.expired else contract.nte
            contract.accruer_annual_leaves = contract.accruer.annual_leaves(deadline)
            contract.accruer_annual_leaves_taken = contract.accruer.annual_leaves_taken(deadline)
            contract.accruer_annual_leaves_balance = contract.accruer.annual_leaves_balance(deadline)

            contract.accruer_sick_leaves = contract.accruer.sick_leaves(deadline)
            contract.accruer_sick_leaves_balance = contract.accruer.sick_leaves_balance(deadline)

            contract.accruer_cto_leaves_taken = contract.accruer.cto_leaves_taken(deadline)
            contract.accruer_cto_leaves_balance = contract.accruer.cto_leaves_balance(deadline)
            contract.accruer_paternity_maternity_leaves_taken = contract.accruer.paternity_maternity_leaves_taken(deadline)
        return _all


class SalaryAdvancePostProcessor(PostProcessor):

    def all(self):
        deadline = datetime.datetime.today()
        _all = self._clone().filter(withhold_payroll=False, nte__gte=deadline)
        toremove = []
        for contract in _all:
            contract.advance_initial_amount = - contract.advancebalance_set.latest_advance_or_zero()
            if contract.advance_initial_amount:
                # FIXME horrible I kwnow. Probably it's time to cleanup things inside the manager
                try:
                    last_advance = contract.advancebalance_set.advances().latest('date')
                    filters = {'date__gt': last_advance.date}
                except contract.DoesNotExist:
                    filters = {}

                contract.advanced_payback_so_far = contract.advancebalance_set.filter(**filters).aggregate(Sum('amount'))['amount__sum'] or 0
                contract.advance_remaning_amount = contract.advancebalance_set.balance()
                contract.advance_balance_after_payroll = max(contract.advance_remaning_amount - contract.advance_next_return_amount, 0)
            else:
                toremove.append(contract.pk)

        return filter(lambda x: x.id not in toremove, _all)


class SalaryOutsideBandPostProcessor(PostProcessor):

    def all(self):
        return(salary for salary in self._clone() if salary.is_outside_band)


registry['PostProcessor'] = PostProcessor
registry['SubTotalPostProcessor'] = SubTotalPostProcessor
registry['ContractPostProcessor'] = ContractPostProcessor
registry['SalaryAdvancePostProcessor'] = SalaryAdvancePostProcessor
registry['LeavePostProcessor'] = LeavePostProcessor
registry['SalaryOutsideBandPostProcessor'] = SalaryOutsideBandPostProcessor





# -*- coding: utf-8 -*-

from django.utils.translation import gettext as _
from ereports.utils import currencyformat


class ColumnWidget(object):
    @classmethod
    def render(cls, raw_value):
        ret = raw_value.value
        if ret is None:
            return ""
        return ret


class YesNoWidget(ColumnWidget):
    @classmethod
    def render(cls, raw_value):
        return _("Yes") if raw_value.value else _("No")


class CurrencyWidget(ColumnWidget):
    thousand_sep = ','
    decimal_sep = '.'

    @classmethod
    def render(cls, raw_value):
        return currencyformat(raw_value.value)


class PercentWidget(ColumnWidget):
    @classmethod
    def render(cls, raw_value):
        return "%s %%" % raw_value.value


class DateWidget(ColumnWidget):
    @classmethod
    def render(cls, raw_value):
        if raw_value.value is not None:
            return raw_value.value.strftime(raw_value.column.format)
        return ""

    @classmethod
    def render_xls(cls, raw_value):
        if raw_value.value is not None:
            return raw_value.value
        return ""


class TimeWidget(ColumnWidget):
    @classmethod
    def render(cls, raw_value):
        if raw_value.value is not None:
            return raw_value.value.strftime(raw_value.column.format)
        return ""



# -*- coding: utf-8 -*-
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.test.testcases import TestCase
from .models import Report

class EasyReportsTestCase(TestCase):
    def test_success(self):
        t = Report(name='Report1',
            target_model=ContentType.objects.get_for_model(User),
            published=False,
            headers='User',
            columns='username',
            filtering='',
            ordering=''
        )
        t.save()

    def test_wrong_filter(self):
        t = Report(name='Report1',
            target_model=ContentType.objects.get_for_model(User),
            published=False,
            headers='',
            columns='',
            filtering='aa',
            ordering=''
        )
        self.assertRaises(ValidationError, t.clean)

    def test_wrong_order(self):
        t = Report(name='Report1',
            target_model=ContentType.objects.get_for_model(User),
            published=False,
            headers='',
            columns='',
            filtering='',
            ordering='qqq'
        )
        self.assertRaises(ValidationError, t.clean)

#    def test_wrong_field(self):
#        t = Report(name='Report1',
#            target_model = ContentType.objects.get_for_model(User),
#            published = False,
#            headers = '',
#            columns = 'nofield',
#            filtering = '',
#            ordering = ''
#        )
#        self.assertRaises(ValidationError, t.clean)



from django.forms import ChoiceField, CharField

from ereports.engine.config import ConfigurationForm, DateRangeField, FilterSpec
from ereports.engine.datasource import Datasource
from ereports.engine.renderer import BaseHtmlRender, BaseXlsRender
from ereports.engine.report import BaseReport
from ereports.tests.app.models import SimpleDemoModel, SimpleDateModel


class SimpleDemoModelSource(Datasource):
    model = SimpleDemoModel
    columns = [
        'char',
        'integer1'
    ]


class SimpleDemoReport(BaseReport):
    datasource = SimpleDemoModelSource.as_datasource()
    list_filter = ['integer1']
    list_display = [
        'char',
    ]
    formats = [
        ('html', BaseHtmlRender),
        ('xls', BaseXlsRender),
    ]


class SimpleDemoExtraForm(ConfigurationForm):
    _extras_data = CharField(max_length=20)


class SimpleDemoExtraReport(SimpleDemoReport):
    config_form_class = SimpleDemoExtraForm
    list_filter = ['integer1', '_extras_data']


class SimpleDateModelSource(Datasource):
    model = SimpleDateModel
    columns = [
        'char',
        'date',
        'date_range',
    ]


class SimpleDateConfigForm(ConfigurationForm):
    date_range = DateRangeField(label='Date Range')
    _report_group_by = ChoiceField()


class SimpleDateReport(BaseReport):
    datasource = SimpleDateModelSource.as_datasource()
    list_filter = ['date']
    group_by = ['char', 'date']
    config_form_class = SimpleDateConfigForm
    forms = [
        ('html', BaseHtmlRender),
        ('xls', BaseXlsRender),
    ]


class FilterSpecField(CharField):
    filter_spec = FilterSpec


class FilterSpecConfigForm(ConfigurationForm):
    date_range = FilterSpecField(label='Date Range')


class FilterSpecDemoReport(BaseReport):
    datasource = SimpleDemoModelSource.as_datasource()
    list_filter = ['date_range']
    config_form_class = FilterSpecConfigForm



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'ReportGroup'
        db.create_table('ereports_reportgroup', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=100)),
            ('parent', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['ereports.ReportGroup'], null=True, blank=True)),
        ))
        db.send_create_signal('ereports', ['ReportGroup'])

        # Adding model 'ReportTemplate'
        db.create_table('ereports_reporttemplate', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=100)),
            ('body', self.gf('django.db.models.fields.TextField')()),
            ('system', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('ereports', ['ReportTemplate'])

        # Adding model 'ReportConfiguration'
        db.create_table('ereports_reportconfiguration', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=100)),
            ('group', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['ereports.ReportGroup'])),
            ('template', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['ereports.ReportTemplate'], null=True, blank=True)),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('report_class', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('target_model', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'], null=True, blank=True)),
            ('published', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('select_related', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('use_distinct', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('columns', self.gf('django.db.models.fields.TextField')(default='id\n')),
            ('filtering', self.gf('django.db.models.fields.TextField')(default='', null=True, blank=True)),
            ('ordering', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('groupby', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('ttl', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('cache_key', self.gf('django.db.models.fields.CharField')(default='U34YSZXDI393H10EZDWEDVK9LVAU2BUWV17888ENWU1Z7UMYGTWY4261LWH6XV1BWX2C6PQ0PFCA5H1RD4WPTTFK2VF8EAVHPQ1W', unique=True, max_length=200)),
        ))
        db.send_create_signal('ereports', ['ReportConfiguration'])


    def backwards(self, orm):
        # Deleting model 'ReportGroup'
        db.delete_table('ereports_reportgroup')

        # Deleting model 'ReportTemplate'
        db.delete_table('ereports_reporttemplate')

        # Deleting model 'ReportConfiguration'
        db.delete_table('ereports_reportconfiguration')


    models = {
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'ereports.reportconfiguration': {
            'Meta': {'object_name': 'ReportConfiguration'},
            'cache_key': ('django.db.models.fields.CharField', [], {'default': "'WOCHS0RMMDG85YZ8DXUFK6X0Q76AL2O7I5OWGLSDFTXP3HZW76HP93WRBZCFWCVCZMSREQ3J5M91FSH2Q5RU39M1KZAM5YE3RE5W'", 'unique': 'True', 'max_length': '200'}),
            'columns': ('django.db.models.fields.TextField', [], {'default': "'id\\n'"}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'filtering': ('django.db.models.fields.TextField', [], {'default': "''", 'null': 'True', 'blank': 'True'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ereports.ReportGroup']"}),
            'groupby': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'ordering': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'report_class': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'select_related': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'target_model': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            'template': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ereports.ReportTemplate']", 'null': 'True', 'blank': 'True'}),
            'ttl': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'use_distinct': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        'ereports.reportgroup': {
            'Meta': {'object_name': 'ReportGroup'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ereports.ReportGroup']", 'null': 'True', 'blank': 'True'})
        },
        'ereports.reporttemplate': {
            'Meta': {'object_name': 'ReportTemplate'},
            'body': ('django.db.models.fields.TextField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'system': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        }
    }

    complete_apps = ['ereports']


# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'ReportConfiguration.description'
        db.alter_column(u'ereports_reportconfiguration', 'description', self.gf('django.db.models.fields.CharField')(max_length=2000, null=True))

    def backwards(self, orm):

        # Changing field 'ReportConfiguration.description'
        db.alter_column(u'ereports_reportconfiguration', 'description', self.gf('django.db.models.fields.CharField')(max_length=255, null=True))

    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'ereports.reportconfiguration': {
            'Meta': {'object_name': 'ReportConfiguration'},
            'cache_key': ('django.db.models.fields.CharField', [], {'default': "'XKJO8Z83SF6UTT5TAHRPHEV1RYI8O2P3N9WSXDN213I6BEZX11KDKIXOJG37FCXG2ERMZ1DMR0Z456PQAOB04FQNMPCHVODXHR1H'", 'unique': 'True', 'max_length': '200'}),
            'columns': ('django.db.models.fields.TextField', [], {'default': "'id\\n'"}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '2000', 'null': 'True', 'blank': 'True'}),
            'filtering': ('django.db.models.fields.TextField', [], {'default': "''", 'null': 'True', 'blank': 'True'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ereports.ReportGroup']"}),
            'groupby': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'ordering': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'report_class': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'select_related': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'target_model': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            'template': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ereports.ReportTemplate']", 'null': 'True', 'blank': 'True'}),
            'ttl': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'use_distinct': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        'ereports.reportgroup': {
            'Meta': {'object_name': 'ReportGroup'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ereports.ReportGroup']", 'null': 'True', 'blank': 'True'})
        },
        'ereports.reporttemplate': {
            'Meta': {'object_name': 'ReportTemplate'},
            'body': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'system': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        }
    }

    complete_apps = ['ereports']



from django.core.exceptions import ValidationError


class FilterDefinitionError(ValidationError):
    def __init__(self, message, code=None, params=None):
        self.message = message
        super(FilterDefinitionError, self).__init__(message, code, params)

    def __str__(self):
        return str(self.message)

    def __repr__(self):
        return str(self.message)


class ColumnDefinitionError(ValidationError):
    pass


class OrderDefinitionError(ValidationError):
    pass


class GroupDefinitionError(ValidationError):
    pass



# -*- coding: utf-8 -*-
from django.http import HttpResponse
from django.template import RequestContext
from ereports.engine.mixins import TemplateRender, ReportRender, XlsRender

DEFAULT_FILENAME = 'report'


class BaseHtmlRender(ReportRender, TemplateRender):
    template_name = 'ereports/tpls/report.html'
    grouped_template_name = 'ereports/tpls/grouped.html'

    def render(self, request, context, **response_kwargs):

        if self.report.group_by:
            context['groups'] = self.report.get_groups()
            self.active_template_name = self.grouped_template_name
        else:
            self.active_template_name = self.template_name

        context.update(self.get_extra_context())
        return super(BaseHtmlRender, self).render(request, context)

    def render_to_response(self, request, **response_kwargs):
        context = RequestContext(request, response_kwargs)
        return HttpResponse(self.render(request, context))

    def get_extra_context(self):
        if self.report.column_totals:
            return {'SUMMARY_FIELDS': filter(lambda x: x in self.report.display_order(), self.report.column_totals)}
        return {}


class BaseXlsRender(ReportRender, XlsRender):
    def render_to_response(self, request, **response_kwargs):
        filename = "{filename}.xls".format(filename=response_kwargs.get("report_name", DEFAULT_FILENAME))
        response = HttpResponse(content_type=self.content_type)
        response['Content-Disposition'] = 'attachment;filename="%s"' % filename
        # ensure we have report context
        if 'report' not in response_kwargs:
            response_kwargs['report'] = self.report
        body = self.render(request, response_kwargs)

        response.write(body)
        return response



# -*- coding: utf-8 -*-
from contextlib import contextmanager
import functools
import random
from django.contrib.auth.models import Permission, Group
from django.core.urlresolvers import reverse
from django.db.models import Q
from django_dynamic_fixture import G
from django_webtest import WebTest
from pasportng.pasport.models import PEmployee, PUser
from pasportng.pasport.models.office import PasportOffice
from pasportng.wings.models import Country
from wfp_djangolib.utils.test import rnd
from wfp_djangolib.utils.test.any import R
from wfp_djangolib.utils.test.rnd import text, unique


def wfp_user(password=None, permissions=[], groups=[], **kwargs):
    is_active = kwargs.pop('is_active', True)
    is_superuser = kwargs.pop('is_superuser', False)
    is_staff = kwargs.pop('is_staff', False)

    user = G(PUser, is_active=is_active, is_superuser=is_superuser,
             is_staff=is_staff, **kwargs)

    for group_name in groups:
        group = Group.objects.get(name=group_name)
        user.groups.add(group)

    for permission_name in permissions:
        app_label, codename = permission_name.split('.')
        permission = Permission.objects.get(
            content_type__app_label=app_label,
            codename=codename)
        user.user_permissions.add(permission)

    if password:
        user.set_password(password)

    user.save()
    return user


def wfp_group(name=None, permissions=[]):
    group = G(Group, name=(name or text(10)))
    for permission_name in permissions:
        app_label, codename = permission_name.split('.')
        permission = Permission.objects.get(
            content_type__app_label=app_label,
            codename=codename)
        group.permissions.add(permission)
    return group


# wfp_country = functools.partial(G, Country,
#                                 # iso_code=lambda x: rnd.iso2(),
#                                 # iso3_code=lambda x: rnd.iso3(),
#                                 num_code=lambda x: rnd.isonum(),
#                                 fullname=lambda x: rnd.text(10),
#                                 name=lambda x: rnd.text(10))
#

wfp_office = functools.partial(G, PasportOffice,
                               # country=lambda x: R(Country, u=True),
                               # type=lambda x: choice(PasportOffice.TYPE)[0],
                               code=lambda x: unique(rnd.office_code)(),
                               name=lambda x: rnd.name())


@contextmanager
def user_add_permission(user, permissions):
    """
        temporary add permissions to user

    :param user:
    :param permissions: list
    :return:

    >>> u = PUser()
    >>> with user_add_permission(u, ['test']):
    ...     u.has_perm('test')
    True
    """
    old = Permission.objects.select_related('user', 'user__groups').filter(Q(user=user) | Q(group__in=user.groups.all()))
    if hasattr(user, '_perm_cache'):
        del user._perm_cache
    new_perms = []
    for perm_name in permissions:
        app_label, code = perm_name.split('.')

        perm = Permission.objects.get(codename=code, content_type__app_label=app_label)
        user.user_permissions.add(perm)
        new_perms.append(perm)
    yield
    for p in new_perms:
        user.user_permissions.remove(p)
    del user._perm_cache


def wfp_pasport_employee(**kwargs):
    """
        wfpemployee(offices=[(Office1, SC),(Office2, SS)])

    :param kwargs:
    :return:
    """
    n = kwargs.pop('n', 1)
    created = []
    for i in range(n):
        params = {'index_no': lambda x: rnd.num(10),
                  'last_name': lambda x: rnd.name(),
                  'first_name': lambda x: rnd.name(),
                  'nickname': lambda x: rnd.text(10),
                  'email': lambda x: rnd.email()}
        params.update(kwargs)
        offices = params.pop('offices', [])
        e = G(PEmployee, **params)
        created.append(e)
    return created


wfp_contract_type = lambda x: random.choice('SSA', 'SC')


class EreportTest(WebTest):
    fixtures = ['initial_reports.json']

    def setUp(self):
        super(EreportTest, self).setUp()
        # wfp_country(5)
        offices = wfp_office(n=2)
        self.office = offices[0]
        self.no_access_office = offices[1]
        e = wfp_pasport_employee(offices=[(self.office, lambda x: wfp_contract_type(x))], n=10)
        self.employee = e[0]
        self.user = wfp_user()

    def test_nav(self):
        url = reverse('pasport.views.office.index', args=[self.office.code])
        # url = reverse('pasport.views.office.index', args=[self.office.code])
        print 111, url, self.user, type(self.user)
        # url = reverse('ereports.list_office_reports', args=[self.office.code])
        # with user_add_office_permission(self.user, self.office, ['billing.list_employee']):
        res = self.app.get(url, user=self.user)
        res.showbrowser()
        # res = res.click(self.office.name)
        # res = res.click('Employees')
        # res.showbrowser()

        # with user_add_office_permission(self.user, self.office, ['billing.view_employee']):
        # res = res.click(self.employee.index_no)


# -*- coding: utf-8 -*-
from collections import defaultdict
from django import forms
from django.core.exceptions import ValidationError
from django.forms import DateField, DateInput, ChoiceField
from django.utils.datastructures import SortedDict
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _
from django.forms.util import flatatt
from django.utils.encoding import force_unicode
from ereports.utils import get_field_from_path, StartWithList


class MultiChoiceWidget(forms.SelectMultiple):
    class Media:
        js = (
            'ereports/multiselect/js/ui.multiselect2.min.js',
            'ereports/multiselect/js/plugins/localisation/jquery.localisation-min.js',
            'ereports/multiselect/js/plugins/tmpl/jquery.tmpl.1.1.1.min.js',
            'ereports/multiselect/js/plugins/blockUI/jquery.blockUI.min.js',
        )
        css = {'all': ("ereports/multiselect/css/ui.multiselect2.css",)}

    def value_from_datadict(self, data, files, name):
        return data.getlist(name)

    def render(self, name, value, attrs=None, choices=()):
        rendered = super(MultiChoiceWidget, self).render(name, value, attrs)
        return rendered + mark_safe(u'''<script type="text/javascript">
            $(document).ready(function() {
                var elem = $('#id_%(name)s');
                elem.addClass('multiselect');
                elem.multiselect({ animated: null, searchable:false, show: null });
            });
            </script>''' % {'name': name})


class ColumnsChoiceField(forms.MultipleChoiceField):
    widget = MultiChoiceWidget

    def __init__(self, report, required=True, widget=None, label=_('Columns'), initial=None, help_text=None, *args,
                 **kwargs):
        choices = [(report.get_column_by_name(col).name, report.get_column_by_name(col).title) for col in
                   report.display_order()]
        initial = report.display_order()
        super(ColumnsChoiceField, self).__init__(choices, required, widget, label, initial, help_text, *args, **kwargs)


class UIDateInput(DateInput):
    def render(self, name, value, attrs=None):
        attrs['class'] = 'uidate2'
        return super(UIDateInput, self).render(name, value, attrs)


class DateRangeInput(DateInput):
    input_type = 'text'

    def value_from_datadict(self, data, files, name):
        values = []
        for suffix in ('start', 'end'):
            n = "%s_%s" % (name, suffix)
            values.append(data.get(n, None))
        return values

    def render(self, name, values, attrs=None):
        if isinstance(values, (list, tuple)):
            start, end = values
            if start is None:
                start = ''
            if end is None:
                end = ''
        else:
            start, end = '', ''

        attrs['class'] = 'uidate2'
        base_final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
        final_attrs_start = base_final_attrs.copy()
        final_attrs_end = base_final_attrs.copy()

        final_attrs_start['id'] = "%s_start" % base_final_attrs['id']
        final_attrs_start['name'] = "%s_start" % base_final_attrs['name']

        final_attrs_end['id'] = "%s_end" % base_final_attrs['id']
        final_attrs_end['name'] = "%s_end" % base_final_attrs['name']

        if start != '':
            final_attrs_start['value'] = force_unicode(self._format_value(start))
        if end != '':
            final_attrs_end['value'] = force_unicode(self._format_value(end))

        return mark_safe(u'From: <input%s /> To: <input%s />' % (flatatt(final_attrs_start), flatatt(final_attrs_end)))


class DateRange(list):
    pass


class DateRangeField(DateField):
    widget = DateRangeInput
    default_error_messages = {
        'required': _(u'This field is required.'),
        'invalid': _(u'Enter a valid value.'),
    }
    input_formats = ('%d/%m/%Y', '%Y-%m-%d')

    def to_python(self, values):
        try:
            start = super(DateRangeField, self).to_python(values[0])
        except ValidationError:
            raise ValidationError(self.error_messages['invalid'] + ' start')

        try:
            end = super(DateRangeField, self).to_python(values[1])
        except ValidationError:
            raise ValidationError(self.error_messages['invalid'] + ' end')

        return DateRange([start, end])

        # def clean(self, values):
        #     values = self.to_python(values)
        #     self.validate(values)
        #     self.run_validators(values)
        #     filters = {}
        #     if values[0]:
        #         filters['%s__gt' % self.name] = values[0]
        #     if values[1]:
        #         filters['%s__lt' % self.name] = values[1]
        #     return FilterSpec(filters)


class FilterSpec(object):
    def __init__(self, value=None, *args, **kwargs):
        self._value = value or {}

    def get_filter_description(self, field):
        return str(self._value)

    def get_filter(self):
        return self._value

    def __nonzero__(self):
        return bool(self._value)


class ConfigurationForm(forms.Form):
    configure_columns = True

    def get_lookup_fields(self):
        lk = []
        for attr_name in self.base_fields.keys():
            if not attr_name.startswith('_'):
                lk.append(attr_name)
        return StartWithList(lk)

    def get_filters(self):
        filters = []
        kwfilters = {}
        self._filters_summary = {}
        lookup_fields = self.get_lookup_fields()
        if lookup_fields:
            for field_name in lookup_fields:
                value = self.cleaned_data[field_name]
                field = self.fields[field_name]
                if value:
                    if isinstance(value, FilterSpec):
                        f = value.get_filter()
                        if isinstance(f, dict):
                            kwfilters.update(f)
                        else:
                            filters.append(f)
                    elif isinstance(value, DateRange):  # here for backward compatibility
                        if value[0]:
                            kwfilters['%s__gt' % field_name] = value[0]
                        if value[1]:
                            kwfilters['%s__lt' % field_name] = value[1]
                    elif isinstance(value, (list, tuple)):
                        kwfilters['%s__in' % field_name] = value
                    else:
                        kwfilters[field_name] = value

                    try:
                        self._filters_summary[field.label] = value.get_filter_description(field)
                    except AttributeError:
                        self._filters_summary[field.label] = value
        return filters, kwfilters

    def clean__report_group_by(self):
        value = self.cleaned_data['_report_group_by']
        if ',' in value:
            group, internal_order = value.split(',')
            if group:
                return group, internal_order
        return None

    def get_filters_summary(self):
        if not hasattr(self, '_filters_summary'):
            self.get_filters()
        return self._filters_summary

    def get_report_attributes(self):
        prefix = '_report_'
        extras = '_extras_'
        attributes = defaultdict(dict)
        for attr_name in self.base_fields.keys():
            if attr_name.startswith(extras):
                key = attr_name[len(extras):]
                attributes['extras'][key] = self.cleaned_data[attr_name]
            if attr_name.startswith(prefix):
                attributes[attr_name[len(prefix):]] = self.cleaned_data[attr_name]

        return attributes

    def clean__report_order_by(self):
        v = self.cleaned_data['_report_order_by']
        v = map(lambda s: s.strip().replace('.', '__'), v.split(','))
        return v

    def _html_output(self, normal_row, error_row, row_ender, help_text_html, errors_on_separate_row):
        s = SortedDict()
        # for name, field in self.fields.items():
        for name in self._field_display_order:
            # if name in self.fields:
            s[name] = self.fields[name]
        self.fields = s
        return super(ConfigurationForm, self)._html_output(normal_row, error_row, row_ender, help_text_html,
                                                           errors_on_separate_row)


def base_has_attr(bases, name):
    none = object()
    for base in bases:
        attr = getattr(base, name, none)
        if attr is not none:
            return True

    return False


def get_attr_from_bases(bases, name, *args):
    # do not set default if not provided
    # we need to raise the exception if no default value
    if args:
        default = args[0]
    none = object()
    for base in bases:
        attr = getattr(base, name, none)
        if attr is not none:
            return attr

    return default


def get_field_from_bases(bases, name, *args):
    # do not set default if not provided
    # we need to raise the exception if no default value
    if args:
        default = args[0]
    none = object()
    for base in bases:
        # lookup both in pForm and mixins
        field = base.base_fields.get(name, getattr(base, name, none))
        if field is not none:
            return field

    return default


def reportform_factory(report, bases=(ConfigurationForm,), **kwargs):
    name = "%sFilterForm" % report.__class__.__name__
    attrs = {}
    field_display_order = []
    for fieldpath in kwargs.get('filtering', getattr(report, 'list_filter', [])):
        if '=' not in fieldpath:
            field = get_field_from_path(report.model, fieldpath)
            if field:
                field_name = fieldpath.replace('.', '__')
                field_display_order.append(field_name)
                attrs[field_name] = get_field_from_bases(bases, field_name, field.formfield(required=False))
            elif fieldpath.startswith('_extras_'):
                field_display_order.append(fieldpath)
                attrs[fieldpath] = get_field_from_bases(bases, fieldpath, None)

    choice_field_class = getattr(report, 'choice_field_class', None) or ChoiceField

    orders = kwargs.get('order_by', None)
    if orders:
        attrs['_report_order_by'] = choice_field_class(label='Order by', choices=orders)
        field_display_order.append('_report_order_by')

    groupby = kwargs.get('groupby', None)
    if groupby:
        attrs['_report_group_by'] = choice_field_class(label='Group by', choices=groupby)
        field_display_order.append('_report_group_by')
    else:
        groupby = get_field_from_bases(bases, '_report_group_by', None)
        if groupby:
            attrs['_report_group_by'] = groupby
            field_display_order.append('_report_group_by')

    # add report format options
    if report.formats:
        formats = [(f[0], f[0]) for f in report.formats]
        attrs['_format'] = choice_field_class(
            label='Format',
            choices=formats
        )
        field_display_order.append('_format')

    configure_columns = kwargs.get('configure_columns', None)
    if configure_columns or get_attr_from_bases(bases, 'configure_columns', False):
        columns_choice_field_class = getattr(report, 'columns_choice_field', ColumnsChoiceField)
        attrs['_report_list_display'] = columns_choice_field_class(report)
        field_display_order.append('_report_list_display')

    attrs['_report'] = report
    attrs['_field_display_order'] = field_display_order

    return type(name, bases, attrs)



# -*- encoding: utf-8 -*-
from __future__ import unicode_literals
from six import StringIO, u
import os
import datetime
import logging
import unicodecsv as csv
from ho import pisa
from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied
from django.db import models
from django.db.models.loading import get_model
from django.http import HttpResponse, HttpResponseServerError, Http404
from django.shortcuts import render_to_response
from django.template import loader
from django.template.context import get_standard_processors, Context, RequestContext
from django.template.response import TemplateResponse
from django.utils.decorators import method_decorator
from django.utils.encoding import smart_str
from django.views.generic import DetailView
from django.views.generic.base import TemplateView
from django.views.generic.list import ListView
from guardian.shortcuts import get_objects_for_user
from kiowa.utils import get_attr
from kiowa.utils.language import cachedproperty
from pasportng.autocache import cache
from pasportng.pasport.fields import MonthField
from pasportng.pasport.models import PasportOffice, Payslip, Payroll
from pasportng.pasport.views2 import SelectedOfficeMixin
from . import math
from .models import Report
from .templatetags.ereports import _get_field, process_queryset_field


logger = logging.getLogger("security")

PAYROLL_FIELDS_AND_LABELS = [
    {'field': 'contract.employee.index_no', 'label': 'Index', },
    {'field': 'contract.employee.first_name', 'label': 'First Name', },
    {'field': 'contract.employee.last_name', 'label': 'Last Name', },
    {'field': 'contract.type', 'label': 'Contract Type', },
    {'field': 'contract.work_level', 'label': 'Work Level', },
    {'field': 'contract.admin_duty_station', 'label': 'Duty Station', },
    {'field': 'contract.initial_eod', 'label': 'Initial EOD', },
    {'field': 'contract.eod', 'label': 'Contract EOD', },
    {'field': 'contract.nte', 'label': 'NTE', },
    {'field': 'basic_salary', 'label': 'Basic Salary', },
    {'field': 'actual_salary', 'label': 'Actual Salary', },
    {'field': 'social_insurance_wfp', 'label': 'Social Insurance (WFP-Emp)', },
    {'field': 'other_allowance', 'label': 'Other Allowances', },
    {'field': 'overtime_cash_compensation', 'label': 'Overtime', },
    {'field': 'hazard_pays_compensation', 'label': 'DDSS', },
    {'field': 'adjustment_credit', 'label': 'Adjustment Credit', },
    {'field': 'total_earning', 'label': 'Total Earning', },
    {'field': 'staff_association', 'label': 'Staff Association', },
    {'field': 'social_insurance_emp', 'label': 'Social Insurance (Emp-Plan)', },
    {'field': 'health_insurance_emp', 'label': 'Health Insurance (Emp)', },
    {'field': 'other_charges', 'label': 'Other Charges', },
    {'field': 'advance_return_amount', 'label': 'Advance return', },
    {'field': 'adjustment_debit', 'label': 'Adjustment debit', },
    {'field': 'total_deduction', 'label': 'Total Deduction', },
    {'field': 'monthly_bills', 'label': 'Bills', },
    {'field': 'net_salary_without_bills', 'label': 'Net Salary', },
    {'field': 'health_insurance_wfp', 'label': 'Health Insurance (WFP)', },
    {'field': 'social_insurance_wfp_to_plan', 'label': 'Social Insurance (WFP-Plan)', },
    {'field': 'death_disability', 'label': 'Death and Disability', },
    {'field': 'total_wfp_cost', 'label': 'Total Cost', },
    {'field': 'contract.fund_reservation', 'label': 'Fund Reservation', },
    {'field': 'contract.internal_order', 'label': 'Internal order', },
    {'field': 'contract.wbs_element', 'label': 'WBS Element', },
    {'field': 'contract.gl_account', 'label': 'GL Account', },
    {'field': 'contract.cost_category', 'label': 'Cost Category', },
    {'field': 'contract.availability_type', 'label': 'Availability Type', },
    {'field': 'starting_date', 'label': 'Salary Period', },
    {'field': 'payroll.month', 'label': 'Payment Month', },
    {'field': 'payroll.exchange_rate', 'label': 'Exchange Rate', },
]

PAYROLL_FIELDS = [x['field'] for x in PAYROLL_FIELDS_AND_LABELS]
PAYROLL_LABELS = [x['label'] for x in PAYROLL_FIELDS_AND_LABELS]


class ProtectedViewMixin(object):
    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(ProtectedViewMixin, self).dispatch(*args, **kwargs)


class OfficeMixin(object):
    @cachedproperty
    def office(self):
        param = self.kwargs.get('office', None)
        user = self.request.user
        if param:
            try:
                office = PasportOffice.objects.get(pk=int(param))
            except ValueError:
                office = PasportOffice.objects.get(code=param)
            if not self.request.user.is_superuser:
                all_perms = self.request.user.get_all_permissions()
                if not bool(all_perms.intersection(['ereports.run_report', 'pasport.can_administer_pasport'])):
                    logger.error("Error on permissions")
                    raise PermissionDenied("No permissions")

                if user.office != office and not user.aditional_offices.filter(pk=office.pk).exists():
                    logger.error("No access to office: %s", office)
                    raise PermissionDenied("No access to office: %s", office)
        else:
            raise Http404
        return office


class CsvViewMixin(object):
    list_display = []

    def render_to_csv(self, context, **response_kwargs):
        response = HttpResponse(mimetype='text/csv')
        response['Content-Disposition'] = 'attachment;filename="%s.csv"' % self.get_filename()

        writer = csv.writer(response, escapechar="\\", delimiter=',', quotechar='"', quoting=csv.QUOTE_ALL)
        writer.writerow([h for h in context['headers']])
        ctx = RequestContext(self.request, {})
        for obj in context['queryset']:
            ctx['obj'] = obj
            row = []
            for fieldname in context['columns']:
                row.append(process_queryset_field(ctx, obj, fieldname))
            writer.writerow(row)
        return response


class PdfViewMixin(object):
    list_display = []

    def render_to_pdf(self, context, **response_kwargs):
        context = self.get_context_data(pagesize='A0', object_list=self.get_queryset())
        html = loader.render_to_string(self.template_name, context)
        result = StringIO.StringIO()
        pdf = pisa.pisaDocument(StringIO.StringIO(html.encode("UTF-8")), dest=result)
        if not pdf.err:
            response = HttpResponse(mimetype='application/pdf')
            response['Content-Disposition'] = 'attachment;filename="payroll_%(office)s_%(year)s_%(month)s.pdf"' % self.kwargs
            response.write(result.getvalue())
            return response


class ExcelViewMixin(object):
    list_display = []

    def _get_style_for_field(self, fieldname):
        field = _get_field(self.model, fieldname)

        FORMATS = {
            models.fields.DateField: 'DD MMM-YY',
            MonthField: 'DD MMM-YY',
            datetime.datetime.date: 'DD MMM-YY',
            datetime.date: 'DD MMM-YY',
            datetime: 'DD MMM-YY',
            models.fields.DateTimeField: 'DD MMD YY hh:mm',
            models.fields.IntegerField: '#,##',
            models.fields.DecimalField: '#,##0.00',
        }

        default = FORMATS.get(type(field), 'general')
        CUSTOM = {'basic_salary': '#,##0.00'}
        return '', CUSTOM.get(fieldname, default)

    def render_to_xls(self, context, **response_kwargs):
        from xlwt import Workbook, XFStyle, easyxf

        response = HttpResponse(mimetype='application/vnd.ms-excel')
        response['Content-Disposition'] = 'attachment;filename="%s.xls"' % self.get_filename()

        w = Workbook(encoding='utf-8')
        w.owner = b'UN World Food Programme'

        ws = w.add_sheet('Payroll')
        style = XFStyle()

        row = 0
        heading_xf = easyxf('font:height 200; font: bold on; align: wrap on, vert centre, horiz center')
        ws.write(row, 0, '#', style)

        for col, fieldname in enumerate(context['headers'], start=1):
            ws.write(row, col, fieldname, heading_xf)
            ws.col(col).width = 5000
        ws.row(row).height = 500

        # we have to prepare all the styles before going into the loop
        # to avoid the "More than 4094 XFs (styles)" Error
        styles = {fieldname: easyxf(*self._get_style_for_field(fieldname)) for fieldname in context['columns']}
        ctx = RequestContext(self.request, {})
        for row, obj in enumerate(context['queryset'], start=row + 1):
            ws.write(row, 0, row)
            ctx['obj'] = obj
            for col, fieldname in enumerate(context['columns'], start=1):
                value = process_queryset_field(ctx, obj, fieldname, raw=True)
                style = styles[fieldname]
                try:
                    ws.write(row, col, value, style=style)
                except Exception:
                    ws.write(row, col, smart_str(value), style=style)

        w.save(self.get_filename())
        response.write(open(self.get_filename()).read())

        return response


class OfficeList(ListView):
    model = PasportOffice
    template_name = 'ereports/payroll/office.html'


class PayrollTemplateResponse(TemplateResponse):
    def __init__(self, request, template, context=None, mimetype=None, status=None, content_type=None, current_app=None):
        super(PayrollTemplateResponse, self).__init__(request, template, context, mimetype, status, content_type, current_app)

    @property
    def rendered_content(self):
        template = self.resolve_template(self.template_name)
        context = self.resolve_context(self.context_data)
        content = template.render(context)
        return content


class PayrollView(SelectedOfficeMixin, CsvViewMixin, ExcelViewMixin, OfficeMixin, ListView):
    """ Payroll report in XLS, CSV, HTML
    """
    template_name = 'ereports/payroll/payroll2.html'
    permissions = []
    model = Payslip
    context_object_name = "payslips"
    allow_empty = True
    list_display = PAYROLL_FIELDS
    response_class = PayrollTemplateResponse

    def get_filename(self):
        import tempfile

        return os.path.join(tempfile.gettempdir(), 'payroll_%(office)s_%(year)s_%(month)s.%(format)s' % self.kwargs)

    def get_cache_key(self, prefix='qs'):
        return "%s_%s" % (prefix, ('payroll_%(office)s_%(year)s_%(month)s_%(format)s' % self.kwargs))

    def get_queryset(self):
        office = self.selected_office
        month = self.kwargs['month']
        year = self.kwargs['year']
        key = self.get_cache_key()
        self.payroll = Payroll.objects.get(office=office, month__month=month, month__year=year)

        if key in cache.cache:
            ret = cache.cache.get(key, version=1)
        else:
            ret = Payslip.nopartial.select_related().filter(payroll=self.payroll)
        cache.cache.set(key, ret, cache.YEAR)
        return ret

    def render_to_response(self, context, **response_kwargs):
        format = self.kwargs.get('format', 'html')
        SUMMARY_FIELDS = ['social_insurance_wfp',
                          'other_allowance', 'overtime_cash_compensation', 'hazard_pays_compensation', 'adjustment_credit',
                          'actual_salary',
                          'total_earning', 'staff_association', 'social_insurance_emp', 'health_insurance_emp', 'other_charges',
                          'advance_return_amount', 'adjustment_debit', 'total_deduction', 'monthly_bills',
                          'net_salary_without_bills', 'health_insurance_wfp', 'social_insurance_wfp_to_plan', 'death_disability',
                          'total_wfp_cost']

        SUBTOTAL_FIELDS = SUMMARY_FIELDS
        context.update({'queryset': self.get_queryset().order_by('contract__actual_duty_station__name'),
                        'headers': PAYROLL_LABELS,
                        'payroll': self.payroll,
                        'SUMMARY_FIELDS': SUMMARY_FIELDS,
                        'SUBTOTAL_FIELDS': SUBTOTAL_FIELDS,
                        'columns': PAYROLL_FIELDS})

        regrouped = math.Regrouper()
        totals = math.Totalizer()
        if format == 'html':
            for record in self.get_queryset():
                ds = record.contract.actual_duty_station.name
                regrouped[ds].append(record)

                for f in SUMMARY_FIELDS:
                    attr = get_attr(record, f)
                    if callable(attr):
                        totals[f].append(attr())
                    else:
                        totals[f].append(attr)

            context.update({'regrouped': regrouped.items(),
                            'totals': totals})

            return self.response_class(request=self.request,
                                       template=self.get_template_names(),
                                       context=context,
                                       **response_kwargs)
        elif format == 'csv':
            return self.render_to_csv(context, **response_kwargs)
        elif format == 'xls':
            return self.render_to_xls(context, **response_kwargs)
            # elif format == 'pdf':
        #     return self.render_to_pdf(context, **response_kwargs)
        return HttpResponseServerError(format)


class ReportIndex(SelectedOfficeMixin, TemplateView):
    """ Index of all available reports for an office
    """
    template_name = 'ereports/index.html'
    permissions = ['ereports.read_report', 'ereports.run_report']

    def get_context_data(self, **kwargs):
        user = self.request.user
        return dict(payrolls=Payroll.objects.select_related().filter(office=self.selected_office).order_by('-month'),
                    reports=get_objects_for_user(user, 'ereports.run_report', Report).filter(published=True).order_by('name'))


class ReportConfigView(SelectedOfficeMixin, DetailView):
    template_name = 'ereports/configure.html'
    permissions = ['ereports.read_report']
    model = Report
    pk_url_kwarg = 'report'


class ReportView(SelectedOfficeMixin, TemplateView, CsvViewMixin, ExcelViewMixin):
    template_name = 'ereports/report.html'
    permissions = ['ereports.read_report']

    def get_filename(self):
        return self.report.name

    def get_report(self):
        pk = self.kwargs.get('report', None)
        return Report.objects.get(pk=pk)

    def get_context_data(self, **kwargs):
        return kwargs

    def render_to_response(self, context, **response_kwargs):
        if self.report.template:
            template = loader.get_template_from_string(self.report.template.body)
            return HttpResponse(template.render(RequestContext(self.request, context)))
        return super(ReportView, self).render_to_response(context, **response_kwargs)

    def render_to_raw(self, context):
        template = loader.get_template('ereports/raw.html')
        return HttpResponse(template.render(RequestContext(self.request, context)), mimetype='text/plain')

    def get(self, request, *args, **kwargs):
        format = self.kwargs.get('format', 'html')
        self.report = self.get_report()
        self.model = self.report.target_model.model_class()
        ctx = {}
        for cp in get_standard_processors():
            ctx.update(**cp(request))
        ctx.update(**self.report.get_extra_context())

        cols, titles = self.report.get_columns()

        context = self.get_context_data(
            selected_office=self.selected_office,
            headers=titles,
            model=str(self.model.__name__),
            columns=cols,
            filters=self.report.get_target_filtering(**ctx),
            queryset=self.report.get_target_queryset(**ctx).all(),
            report=self.report
        )

        if format == 'html':
            return self.render_to_response(context)
        elif format == 'csv':
            return self.render_to_csv(context)
        elif format == 'xls':
            return self.render_to_xls(context)
        # elif format == 'pdf':
        #     return self.render_to_pdf(context)
        elif format == 'raw':
            return self.render_to_raw(context)
        return HttpResponseServerError(format)


def get_model_meta(request, owner, app_name, model_name):
    model = get_model(app_name, model_name)
    fields = sorted(model._meta.fields, key=lambda x: x.verbose_name)
    return render_to_response('ereports/meta_info.html', Context({'owner': owner, 'fields': fields}))



from django.contrib.auth.models import Permission
from django.core.exceptions import ImproperlyConfigured
from django.contrib.auth.models import User
from django.test.testcases import TestCase
from django_dynamic_fixture import G
import itertools
import mock
from ereports.engine.cache import DummyCacheManager, DatasourceCacheManager
from ereports.engine.columns import Column, CalcColumn, ColumnCallable, BooleanColumn, OptionalColumn
from ereports.engine.datasource import Datasource, RecordFilteredError
from ereports.tests import app
from ereports.tests.app.models import SimpleDemoModel, DemoOptionalModel
from ereports.utils import get_verbose_name


def test_get_verbose_name():
    l = get_verbose_name(SimpleDemoModel, 'char')
    assert l == 'Character'


class TestDatasource(TestCase):
    def test_inherit(self):
        TestDatasource = type('TestDatasource', (Datasource,), {'model': Permission, 'columns': ['id']})
        ds = TestDatasource.as_datasource()
        self.assertIsInstance(list(ds), list)

    def test_inherit_exception(self):
        TestDatasource = type('TestDatasource', (Datasource, ), {'model': Permission, 'columns': ['id']})
        with self.assertRaises(TypeError):
            kwargs = dict(wrong='oops')
            TestDatasource.as_datasource(**kwargs)

    def test_inherit_improperlyconfigured(self):
        TestDatasource = type('TestDatasource', (Datasource, ), {'columns': ['id']})
        with self.assertRaises(ImproperlyConfigured):
            kwargs = dict(model=None)
            TestDatasource.as_datasource(**kwargs)

    def test_create_from_model(self):
        instances = G(SimpleDemoModel, n=2, char='1', integer1=1, integer2=2, boolean=True)

        self.assertEquals(len(instances), 2)

        ds = Datasource.as_datasource(model=SimpleDemoModel)

        self.assertSequenceEqual([c.name for c in ds.columns], ['id', 'char', 'integer1', 'integer2', 'boolean'])
        self.assertSequenceEqual([c.title for c in ds.columns],
                                 ['ID', 'Character', 'Integer #1', 'Integer #2', 'Boolean'])
        self.assertSequenceEqual(ds, [(instances[0].pk, u'1', 1, 2, True), (instances[1].pk, u'1', 1, 2, True)])

    def test_columns(self):
        G(SimpleDemoModel, n=2, char='abc', integer1=1)
        ds = Datasource.as_datasource(model=SimpleDemoModel,
                                      columns=[Column('char', 'AAA'), Column('integer1')])

        self.assertSequenceEqual([c.name for c in ds.columns], ['char', 'integer1'])
        self.assertSequenceEqual([c.title for c in ds.columns], ['AAA', 'Integer #1'])
        self.assertSequenceEqual(ds, [(u'abc', 1), (u'abc', 1)])

    def test_list_columns(self):
        G(SimpleDemoModel, n=2, char='abc', integer1=1, integer2=3)
        ds = Datasource.as_datasource(model=SimpleDemoModel,
                                      columns=[('char', Column), ('integer1', Column)])
        self.assertSequenceEqual([c.name for c in ds.columns], ['char', 'integer1'])
        self.assertSequenceEqual([c.title for c in ds.columns], ['Character', 'Integer #1'])
        self.assertSequenceEqual(ds, [(u'abc', 1), (u'abc', 1)])

    def test_manipulator(self):
        G(SimpleDemoModel, n=2, char='abc', integer1=1)
        ds = Datasource.as_datasource(model=SimpleDemoModel,
                                      columns=[Column('char', manipulator=lambda v: v.upper())])
        self.assertSequenceEqual(ds, [(u'ABC', ), (u'ABC', )])

    def test_custom_column_from_string(self):
        G(SimpleDemoModel, n=2, char='abc', integer1=1, integer2=3)
        ds = Datasource.as_datasource(model=SimpleDemoModel,
                                      extra_column=lambda obj: 'extra_value',
                                      columns=[Column('integer1'),
                                               'integer2',
                                               CalcColumn(['integer1', 'integer2'])])
        self.assertSequenceEqual(ds, [(1, 3, 4), (1, 3, 4)])

    def test_create_from_queryset(self):
        instances = G(SimpleDemoModel, n=2, char='abc')

        self.assertEquals(len(instances), 2)

        ds = Datasource.as_datasource(queryset=SimpleDemoModel.objects.all(),
                                      columns=['id', 'char'])

        self.assertSequenceEqual([c.name for c in ds.columns], ['id', 'char'])
        self.assertSequenceEqual([c.title for c in ds.columns], ['ID', 'Character'])
        self.assertSequenceEqual(ds, [(instances[0].pk, u'abc'), (instances[1].pk, u'abc')])

    def test_get_col_by_name(self):
        G(SimpleDemoModel, n=2, char='abc')
        ds = Datasource.as_datasource(queryset=SimpleDemoModel.objects.all())
        self.assertSequenceEqual([u'abc', u'abc'], [row.char for row in ds])
        self.assertSequenceEqual([u'abc', u'abc'], [row['char'] for row in ds])

    def test_filter_queryset(self):
        instances = G(SimpleDemoModel, n=5, char='abc')

        self.assertEquals(len(instances), 5)

        ds = Datasource.as_datasource(queryset=SimpleDemoModel.objects.all(),
                                      columns=['id', 'char'])
        ds.add_filters(id__gt=instances[2].pk)
        self.assertSequenceEqual(ds, [(instances[3].pk, u'abc'), (instances[4].pk, u'abc')])

    def test_post_filter(self):
        instances = G(SimpleDemoModel, n=5, char='abc')
        ds = Datasource.as_datasource(queryset=SimpleDemoModel.objects.all(),
                                      columns=['id', 'char'])
        ds.add_filters(id__gt=instances[2].pk)
        self.assertSequenceEqual(ds, [(instances[3].pk, u'abc'), (instances[4].pk, u'abc')])

    def test_cachemanager(self):
        ds = Datasource.as_datasource(model=SimpleDemoModel, use_cache=False)
        self.assertIsInstance(ds.cache_manager, DummyCacheManager)

        ds = Datasource.as_datasource(model=SimpleDemoModel, use_cache=True)
        self.assertIsInstance(ds.cache_manager, DatasourceCacheManager)

    # def test_internal_cache(self):
    #     G(SimpleDemoModel, n=10, char='abc')
    #     ds = Datasource.as_datasource(model=SimpleDemoModel, columns=[Column('filter_source')])

    def test_custom_filter(self):
        G(SimpleDemoModel, n=10, char='abc')
        app.models.counter = itertools.count()
        ds = Datasource.as_datasource(model=SimpleDemoModel, columns=[Column('filter_source')])
        ds._get_queryset = mock.Mock(wraps=ds._get_queryset)
        ds._create_result_cache = mock.Mock(wraps=ds._create_result_cache)

        def filter_odd(row):
            if not row.filter_source.value % 2:
                raise RecordFilteredError

        ds.add_custom_filter(filter_odd)
        list(ds)
        list(ds)
        self.assertSequenceEqual(ds, [(1,), (3,), (5,), (7,), (9,)])
        assert ds._get_queryset.call_count == 1
        assert ds._create_result_cache.call_count == 1


class TestColumns(TestCase):
    def test_custom_column_callable(self):
        G(SimpleDemoModel, n=2, char='abc', integer1=10, integer2=20)

        def _custom_callable(obj, ds):
            return obj.integer1 + 100

        ds = Datasource.as_datasource(model=SimpleDemoModel,
                                      columns=[Column('integer1'),
                                               Column('integer2'),
                                               ColumnCallable(_custom_callable)])

        self.assertSequenceEqual(ds, [(10, 20, 110), (10, 20, 110)])

    def test_custom_column(self):
        G(SimpleDemoModel, n=2, char='abc', integer1=1, integer2=3)
        ds = Datasource.as_datasource(model=SimpleDemoModel,
                                      columns=[Column('integer1'), Column('integer2'),
                                               CalcColumn(['integer1', 'integer2'])])
        self.assertSequenceEqual(ds, [(1, 3, 4), (1, 3, 4)])

    def test_custom_column_from_datasource_method(self):
        G(SimpleDemoModel, n=2, char='abc', integer1=1, integer2=3)
        ds = Datasource.as_datasource(model=SimpleDemoModel,
                                      extra_column=lambda obj: 'extra_value',
                                      columns=[Column('integer1'),
                                               Column('integer2'),
                                               Column('extra_column'),
                                               CalcColumn(['integer1', 'integer2'])])
        self.assertSequenceEqual(ds, [(1, 3, 'extra_value', 4), (1, 3, 'extra_value', 4)])

    def test_boolean_column_no(self):
        G(SimpleDemoModel, n=2, boolean=False)
        ds = Datasource.as_datasource(model=SimpleDemoModel,
                                      columns=[BooleanColumn('boolean')])
        self.assertSequenceEqual([c.title for c in ds.columns], ['Boolean'])
        self.assertSequenceEqual(ds, [(False,), (False,)])

    def test_booleanyesno_column_yes(self):
        G(SimpleDemoModel, n=2, boolean=True)
        ds = Datasource.as_datasource(model=SimpleDemoModel,
                                      columns=[BooleanColumn('boolean')])
        self.assertSequenceEqual([c.title for c in ds.columns], ['Boolean'])
        self.assertSequenceEqual(ds, [(True,), (True,)])

    def test_optional_column(self):
        G(DemoOptionalModel, n=2, name='abc', user=G(User, n=1, first_name='user1'))
        ds = Datasource.as_datasource(model=DemoOptionalModel,
                                      columns=[Column('name'), OptionalColumn('user.first_name')])
        self.assertSequenceEqual([c.title for c in ds.columns], ['name', 'first name'])
        self.assertSequenceEqual(ds, [('abc', 'user1',), ('abc', 'user1')])

    def test_optional_column_optional(self):
        G(DemoOptionalModel, n=2, name='abc', user=None)
        ds = Datasource.as_datasource(model=DemoOptionalModel,
                                      columns=[Column('name'), OptionalColumn('user.first_name')])
        self.assertSequenceEqual([c.title for c in ds.columns], ['name', 'first name'])
        self.assertSequenceEqual(ds, [('abc', '',), ('abc', '')])



# -*- encoding: utf-8 -*-
import json
import random
import string
import datetime
from dateutil.relativedelta import relativedelta
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError, FieldError, ObjectDoesNotExist
from django.db import models
from django.db.models import permalink
from django.template.base import Variable, VariableDoesNotExist, Template
from django.template.context import Context
from guardian.models import GroupObjectPermission
from pasportng.ereports import postprocessor
from pasportng.ereports.postprocessor import PostProcessor
from pasportng.ereports.exceptions import OrderDefinitionError, FilterDefinitionError, GroupDefinitionError
from pasportng.pasport.utils.date import last_of_month
from .templatetags.ereports import _get_field, header


clean_field = lambda field: field.replace('.', '__').strip()
keygen = ''.join(random.choice(string.ascii_uppercase + string.digits) for x in range(100))


class ReportGroup(models.Model):
    name = models.CharField(max_length=100, unique=True)
    parent = models.ForeignKey('self', blank=True, null=True)

    class Meta:
        app_label = 'ereports'

    def __unicode__(self):
        return self.name


class ReportTemplate(models.Model):
    name = models.CharField(max_length=100, unique=True)
    body = models.TextField()
    system = models.BooleanField(default=False)

    class Meta:
        app_label = 'ereports'

    def __unicode__(self):
        return self.name


class Report(models.Model):
    name = models.CharField(max_length=100, unique=True)
    group = models.ForeignKey(ReportGroup)
    template = models.ForeignKey(ReportTemplate, blank=True, null=True)
    description = models.CharField(max_length=255, blank=True, null=True)

    target_model = models.ForeignKey(ContentType)
    datasource = models.CharField('PostProcessor', max_length=200, blank=True, null=True, choices=[(k, k) for k, v in postprocessor.registry.iteritems()])

    published = models.BooleanField(default=False)
    select_related = models.BooleanField(default=False)
    use_distinct = models.BooleanField(default=False)

    columns = models.TextField(default='id\n')
    filtering = models.TextField(blank=True, null=True, default="office={{selected_office}}")
    ordering = models.TextField(blank=True, null=True)
    groupby = models.TextField(blank=True, null=True)
    extras = models.TextField(blank=True, null=True)
    rawsql = models.TextField(blank=True, null=True)

    ttl = models.IntegerField('Cache validity (minutes)', default=0)
    cache_key = models.CharField('Cache entry seed', max_length=200, null=False, unique=True, default=keygen)

    class Meta:
        permissions = (('run_report', 'Can run a report'),)
        app_label = 'ereports'

    def __unicode__(self):
        return self.name

    @permalink
    def get_absolute_url(self):
        return 'reports2.do_report', [self.pk]

    def save(self, force_insert=False, force_update=False, using=None):
        if self.cache_key is None:
            self.cache_key = '{0.pk} {0.name} {0.filtering}'.format(self)
        super(Report, self).save(force_insert, force_update, using)

    def get_extra(self, key, default=None):
        if not hasattr(self, '_extras'):
            try:
                self._extras = json.loads(self.extras)
            except ValueError:
                self._extras = {}

        return self._extras.get(key, default)

    def get_target_queryset(self, *args, **context):
        Datasource = postprocessor.registry.get(self.datasource, PostProcessor)
        try:
            model = self.target_model.model_class()

            manager_method_name = self.get_extra('manager', 'filter')
            manager_method = getattr(model.objects, manager_method_name, model.objects.filter)

            filters = self.get_target_filtering(*args, **context)
            order = self.get_target_ordering(*args, **context)

            qs = manager_method(**filters).order_by(*order)

            if self.select_related:
                return qs.select_related()
            qs = Datasource(qs.model, qs.query, None, report=self, queryset=qs)
            return qs
        except FieldError as e:
            raise ValidationError(e)
        except ValidationError as e:
            raise e

    # noinspection PyUnusedLocal
    def get_target_ordering(self, *args, **kwargs):
        try:
            order = []
            for line in self.ordering.split('\n'):
                if line.strip():
                    order.append(line.strip().replace('.', '__'))
            return order
        except Exception as e:
            raise OrderDefinitionError(e)

    def _process_value(self, value_string, context):
        if value_string.startswith('{%'):
            # context['current_row'] = obj
            # HACK!!  todo: find a better way to do that
            pattern = "{%% load ereports %%}%s" % value_string
            t = Template(pattern)
            value = t.render(Context(context))
            if value != u'None':
                return value
        elif value_string.startswith('{{'):
            target = Variable(value_string[2:-2].strip())
            try:
                return target.resolve(Context(context))
            except VariableDoesNotExist:
                return None
        else:
            return value_string

    # noinspection PyUnusedLocal
    def get_target_filtering(self, *args, **kwargs):
        filters = {}
        line = None
        try:
            if self.filtering:
                for line in self.filtering.split('\r\n'):
                    if line.strip():
                        field, value = line.split('=', 2)
                        if field[-1] in '<>':
                            pass
                        v = self._process_value(value, kwargs)
                        if v:
                            filters[clean_field(field)] = v
            return filters
        except Exception as e:
            if line is not None:
                raise FilterDefinitionError("Error on line:`%s` %s" % (line, str(e)))
            raise FilterDefinitionError(e)

    # noinspection PyUnusedLocal
    def get_target_grouping(self, *args, **kwargs):
        groups = []
        line = None
        try:
            if self.groupby:
                for line in self.groupby.split('\r\n'):
                    groups.append(line)
            return groups
        except Exception as e:
            if line is not None:
                raise GroupDefinitionError("Error on line:`%s` %s" % (line, str(e)))
            raise GroupDefinitionError(e)

    def get_columns(self):
        model = self.target_model.model_class()
        cols = []
        headers = []
        for col in self.columns.split('\r\n'):
            parts = col.split('#')
            col = parts[0]
            if len(parts) == 1:
                title = header(model, col)
            elif len(parts) == 2:
                title = parts[1]
            if title.strip():
                headers.append(str(title))
            if col.strip():
                cols.append(col)

        return cols, headers

    # TODO check if this being used
    def clean_target_model(self):
        try:
            # TODO check for dead code
            model = self.target_model.model_class()
        except ObjectDoesNotExist as e:
            raise ValidationError('Invalid Target: %s' % e)

    def get_extra_context(self):
        today = datetime.datetime.today()
        last_day_prev_month = last_of_month(today - relativedelta(months=1))
        # TODO check results for prev_month. Is it intented to be full date or just the month value?
        context = {
            'today': today,
            'current_month': today.month,
            'prev_month': today.month - 1,
            'last_day_prev_month': last_day_prev_month,
            'current_year': today.year,
            'prev_year': today.year - 1
        }
        return context

    def __clean(self):
        """
            * validate filter
            * validate order
            * validate queryset
            * validate header
            * get an object by queryset
            * validate fields
        """
        try:
            model = self.target_model.model_class()
        except ObjectDoesNotExist:
            return

        # filtering
        # try:
        #    for field, value in self.get_target_filtering().items():
        #        if not _get_field(model, field):
        #            raise ValidationError('Invalid Filter: `%s` is not a valid field' % field)
        # except  ValueError as e:
        #    raise ValidationError('Invalid filter: %s' % e)
        #    raise ValidationError('Invalid filter: Entries must be in the formar `field=value`')

        # ordering
        line = None
        try:
            for line in self.get_target_ordering():
                if line.startswith('-'):
                    line = line[1:]
                if not _get_field(model, line):
                    raise ValidationError('Invalid ordering: `%s` is not a valid field ' % line)
        except ValueError:
            raise ValidationError('Invalid ordering: `%s`.' % line)

        # columns
        try:
            self.get_columns()
        except Exception as e:
            raise ValidationError('Invalid column: %s' % e)
        ctx = {}
        today = datetime.datetime.today()
        ctx.update({'today': today,
                    'current_month': today.month,
                    'prev_month': today.month - 1})
        try:
            list(self.get_target_queryset(**self.get_extra_context())[1:3])
        except Exception as e:
            raise ValidationError('Cannot create a queryset. Please check your inputs %s' % str(e))


class ReportPermission(GroupObjectPermission):
    class Meta:
        proxy = True
        app_label = 'ereports'
        verbose_name = 'Report Permission'



# -*- coding: utf-8 -*-
import datetime

from ereports.engine.columns import Column, RowValue
from ereports.engine.widgets import ColumnWidget, YesNoWidget, CurrencyWidget, PercentWidget, DateWidget, TimeWidget


def test_columnwidget():
    c = Column('normal')
    r = RowValue('abc', column=c)
    assert ColumnWidget.render(r) == 'abc'

    r = RowValue(None, column=c)
    assert ColumnWidget.render(r) == ''


def test_yesnowidget():
    c = Column('normal')
    r = RowValue(True, column=c)
    assert YesNoWidget.render(r) == 'Yes'

    r = RowValue('abc', column=c)
    assert YesNoWidget.render(r) == 'Yes'

    r = RowValue(False, column=c)
    assert YesNoWidget.render(r) == 'No'


def test_currencywidget():
    c = Column('normal')
    r = RowValue(1000.00, column=c)
    assert CurrencyWidget.render(r) == "1,000.00"

    r = RowValue(1000, column=c)
    assert CurrencyWidget.render(r) == "1,000.00"


def test_percentwidget():
    c = Column('normal')
    r = RowValue("10", column=c)
    assert PercentWidget.render(r) == "10 %"

    r = RowValue(10, column=c)
    assert PercentWidget.render(r) == "10 %"

    r = RowValue(10.0, column=c)
    assert PercentWidget.render(r) == "10.0 %"

    r = RowValue(10.01, column=c)
    assert PercentWidget.render(r) == "10.01 %"


def test_datewidget():
    c = Column('normal', format="%Y-%m-%d")
    r = RowValue(datetime.date(2000, 1, 1), column=c)
    assert DateWidget.render(r) == "2000-01-01"
    assert DateWidget.render_xls(r) == datetime.date(2000, 1, 1)

    r = RowValue(None, column=c)
    assert DateWidget.render(r) == ""


def test_timewidget():
    c = Column('normal', format="%H:%M")
    r = RowValue(datetime.time(16, 20, 10), column=c)
    assert TimeWidget.render(r) == "16:20"

    r = RowValue(None, column=c)
    assert TimeWidget.render(r) == ""
